@@ -1,3 +1,5 @@
+# PROPHET BOOST ----
+
 #' General Interface for Boosted PROPHET Time Series Models
 #'
 #' `prophet_boost()` is a way to generate a _specification_ of a Boosted PROPHET model
@@ -5,19 +7,7 @@
 #'  different packages. Currently the only package is `prophet`.
 #'
 #' @inheritParams arima_boost
-#' @param mode A single character string for the type of model.
-#'  The only possible value for this model is "regression".
-#' @param growth String 'linear' or 'logistic' to specify a linear or logistic trend.
-#' @param num_changepoints Number of potential changepoints to include for modeling trend.
-#' @param season 'additive' (default) or 'multiplicative'.
-#' @param prior_scale_changepoints Parameter modulating the flexibility of the
-#'  automatic changepoint selection. Large values will allow many changepoints,
-#'  small values will allow few changepoints.
-#' @param prior_scale_seasonality Parameter modulating the strength of the
-#'  seasonality model. Larger values allow the model to fit larger seasonal
-#'  fluctuations, smaller values dampen the seasonality.
-#' @param prior_scale_holidays Parameter modulating the strength of the holiday components model,
-#'  unless overridden in the holidays input.
+#' @inheritParams prophet_reg
 #'
 #'
 #' @details
@@ -34,8 +24,10 @@
 #'
 #' The main arguments (tuning parameters) for the __PROPHET__ model are:
 #'
-#'  - `growth`: String 'linear' or 'logistic' to specify a linear or logistic trend.
-#' - `num_changepoints`: Number of potential changepoints to include for modeling trend.
+#' - `growth`: String 'linear' or 'logistic' to specify a linear or logistic trend.
+#' - `changepoint_num`: Number of potential changepoints to include for modeling trend.
+#' - `changepoint_range`: Range changepoints that adjusts how close to the end
+#'    the last changepoint can be located.
 #' - `season`: 'additive' (default) or 'multiplicative'.
 #' - `prior_scale_changepoints`: Parameter modulating the flexibility of the
 #'   automatic changepoint selection. Large values will allow many changepoints,
@@ -45,6 +37,8 @@
 #'  fluctuations, smaller values dampen the seasonality.
 #' - `prior_scale_holidays`: Parameter modulating the strength of the holiday components model,
 #'  unless overridden in the holidays input.
+#' - `logistic_cap`: When growth is logistic, the upper-bound for "saturation".
+#' - `logistic_floor`: When growth is logistic, the lower-bound for "saturation".
 #'
 #' The main arguments (tuning parameters) for the model __XGBoost model__ are:
 #'
@@ -84,7 +78,7 @@
 #' tibble::tribble(
 #'     ~ "modeltime", ~ "prophet",
 #'     "growth", "growth",
-#'     "num_changepoints", "n.changepoints",
+#'     "changepoint_num", "n.changepoints",
 #'     "season", "seasonality.mode",
 #'     "prior_scale_changepoints", "changepoint.prior.scale",
 #'     "prior_scale_seasonality", "seasonality.prior.scale",
@@ -128,6 +122,11 @@
 #'  uncertainty intervals are not used as part of the Modeltime Workflow.
 #'  You can override this setting if you plan to use prophet's uncertainty tools.
 #'
+#' Logistic Growth and Saturation Levels:
+#' - For `growth = "logistic"`, simply add numeric values for `logistic_cap` and / or
+#'   `logistic_floor`. There is _no need_ to add additional columns
+#'   for "cap" and "floor" to your data frame.
+#'
 #' Limitations:
 #' - `prophet::add_seasonality()` is not currently implemented. It's used to
 #'  specify non-standard seasonalities using fourier series. An alternative is to use
@@ -222,9 +221,12 @@
 #'
 #' @export
 prophet_boost <- function(mode = "regression",
-                          growth = NULL, num_changepoints = NULL, season = NULL,
+                          growth = NULL, changepoint_num = NULL, changepoint_range = NULL,
+                          seasonality_yearly = NULL, seasonality_weekly = NULL, seasonality_daily = NULL,
+                          season = NULL,
                           prior_scale_changepoints = NULL, prior_scale_seasonality = NULL,
                           prior_scale_holidays = NULL,
+                          logistic_cap = NULL, logistic_floor = NULL,
                           mtry = NULL, trees = NULL, min_n = NULL,
                           tree_depth = NULL, learn_rate = NULL,
                           loss_reduction = NULL,
@@ -234,11 +236,17 @@ prophet_boost <- function(mode = "regression",
 
         # Prophet
         growth                    = rlang::enquo(growth),
-        num_changepoints          = rlang::enquo(num_changepoints),
+        changepoint_num           = rlang::enquo(changepoint_num),
+        changepoint_range         = rlang::enquo(changepoint_range),
+        seasonality_yearly        = rlang::enquo(seasonality_yearly),
+        seasonality_weekly        = rlang::enquo(seasonality_weekly),
+        seasonality_daily         = rlang::enquo(seasonality_daily),
         season                    = rlang::enquo(season),
         prior_scale_changepoints  = rlang::enquo(prior_scale_changepoints),
         prior_scale_seasonality   = rlang::enquo(prior_scale_seasonality),
         prior_scale_holidays      = rlang::enquo(prior_scale_holidays),
+        logistic_cap              = rlang::enquo(logistic_cap),
+        logistic_floor            = rlang::enquo(logistic_floor),
 
         # XGBoost
         mtry                      = rlang::enquo(mtry),
@@ -278,9 +286,12 @@ print.prophet_boost <- function(x, ...) {
 #' @export
 #' @importFrom stats update
 update.prophet_boost <- function(object, parameters = NULL,
-                                 growth = NULL, num_changepoints = NULL, season = NULL,
+                                 growth = NULL, changepoint_num = NULL, changepoint_range = NULL,
+                                 seasonality_yearly = NULL, seasonality_weekly = NULL, seasonality_daily = NULL,
+                                 season = NULL,
                                  prior_scale_changepoints = NULL, prior_scale_seasonality = NULL,
                                  prior_scale_holidays = NULL,
+                                 logistic_cap = NULL, logistic_floor = NULL,
                                  mtry = NULL, trees = NULL, min_n = NULL,
                                  tree_depth = NULL, learn_rate = NULL,
                                  loss_reduction = NULL,
@@ -297,11 +308,17 @@ update.prophet_boost <- function(object, parameters = NULL,
 
         # Prophet
         growth                    = rlang::enquo(growth),
-        num_changepoints          = rlang::enquo(num_changepoints),
+        changepoint_num           = rlang::enquo(changepoint_num),
+        changepoint_range         = rlang::enquo(changepoint_range),
+        seasonality_yearly        = rlang::enquo(seasonality_yearly),
+        seasonality_weekly        = rlang::enquo(seasonality_weekly),
+        seasonality_daily         = rlang::enquo(seasonality_daily),
         season                    = rlang::enquo(season),
         prior_scale_changepoints  = rlang::enquo(prior_scale_changepoints),
         prior_scale_seasonality   = rlang::enquo(prior_scale_seasonality),
         prior_scale_holidays      = rlang::enquo(prior_scale_holidays),
+        logistic_cap              = rlang::enquo(logistic_cap),
+        logistic_floor            = rlang::enquo(logistic_floor),
 
         # XGBoost
         mtry                      = rlang::enquo(mtry),
@@ -355,6 +372,7 @@ translate.prophet_boost <- function(x, engine = x$engine, ...) {
 #' Low-Level PROPHET function for translating modeltime to Boosted PROPHET
 #'
 #' @inheritParams prophet::prophet
+#' @inheritParams prophet_boost
 #' @param x A dataframe of xreg (exogenous regressors)
 #' @param y A numeric vector of values to fit
 #' @param max_depth An integer for the maximum depth of the tree.
@@ -391,6 +409,8 @@ prophet_xgboost_fit_impl <- function(x, y,
                                      seasonality.prior.scale = 10,
                                      holidays.prior.scale = 10,
                                      changepoint.prior.scale = 0.05,
+                                     logistic_cap = NULL,
+                                     logistic_floor = NULL,
                                      mcmc.samples = 0,
                                      interval.width = 0.8,
                                      uncertainty.samples = 1000,
@@ -409,6 +429,8 @@ prophet_xgboost_fit_impl <- function(x, y,
     outcome    <- y
     predictor  <- x
 
+    growth <- tolower(growth)
+
     if (!growth[1] %in% c("linear", "logistic")) {
         message("growth must be 'linear' or 'logistic'. Defaulting to 'linear'.")
         growth <- 'linear'
@@ -419,6 +441,12 @@ prophet_xgboost_fit_impl <- function(x, y,
         seasonality.mode <- 'additive'
     }
 
+    if (growth == "logistic") {
+        if (all(c(is.null(logistic_cap), is.null(logistic_floor)))) {
+            glubort("Capacities must be supplied for `growth = 'logistic'`. Try specifying at least one of 'logistic_cap' or 'logistic_floor'")
+        }
+    }
+
     # INDEX & PERIOD
     # Determine Period, Index Col, and Index
     index_tbl <- parse_index_from_data(predictor)
@@ -439,7 +467,14 @@ prophet_xgboost_fit_impl <- function(x, y,
         ds = idx
     )
 
+    # Add logistic cap / floor
+    if (growth == "logistic") {
+        df$cap   <- logistic_cap
+        df$floor <- logistic_floor
+    }
+
     # Construct model
+    # Fit model
     fit_prophet <- prophet::prophet(
         df = df,
         growth = growth,
@@ -460,6 +495,7 @@ prophet_xgboost_fit_impl <- function(x, y,
         fit = fit
     )
 
+
     # In-sample Predictions
     prophet_fitted    <- stats::predict(fit_prophet, df) %>% dplyr::pull(yhat)
     prophet_residuals <- outcome - prophet_fitted
@@ -501,7 +537,12 @@ prophet_xgboost_fit_impl <- function(x, y,
 
     # Extras - Pass on transformation recipe
     extras <- list(
-        xreg_recipe = xreg_recipe
+        xreg_recipe = xreg_recipe,
+        logistic_params = list(
+            growth         = growth,
+            logistic_cap   = logistic_cap,
+            logistic_floor = logistic_floor
+        )
     )
 
     # Model Description - Gets printed to describe the high-level model structure
@@ -521,10 +562,35 @@ prophet_xgboost_fit_impl <- function(x, y,
 #' @export
 print.prophet_xgboost_fit_impl <- function(x, ...) {
 
+    prophet_model <- x$models$model_1
+
+    logistic_params <- x$extras$logistic_params
+    if (is.null(logistic_params$logistic_cap)) {
+        cap <- "NULL"
+    } else {
+        cap <- logistic_params$logistic_cap
+    }
+    if (is.null(logistic_params$logistic_floor)) {
+        floor <- "NULL"
+    } else {
+        floor <- logistic_params$logistic_floor
+    }
+
     msg_1 <- stringr::str_glue(
-         "- growth: '{x$models$model_1$growth}'
-          - n.changepoints: {x$models$model_1$n.changepoints}
-          - seasonality.mode: '{x$models$model_1$seasonality.mode}'")
+         "
+          - growth: '{prophet_model$growth}'
+          - n.changepoints: {prophet_model$n.changepoints}
+          - changepoint.range: {prophet_model$changepoint.range}
+          - yearly.seasonality: '{prophet_model$yearly.seasonality}'
+          - weekly.seasonality: '{prophet_model$weekly.seasonality}'
+          - daily.seasonality: '{prophet_model$daily.seasonality}'
+          - seasonality.mode: '{prophet_model$seasonality.mode}'
+          - changepoint.prior.scale: {prophet_model$changepoint.prior.scale}
+          - seasonality.prior.scale: {prophet_model$seasonality.prior.scale}
+          - holidays.prior.scale: {prophet_model$holidays.prior.scale}
+          - logistic_cap: {cap}
+          - logistic_floor: {floor}
+         ")
 
     if (!is.null(x$desc)) cat(paste0(x$desc,"\n"))
     cat("---\n")
@@ -555,18 +621,22 @@ predict.prophet_xgboost_fit_impl <- function(object, new_data, ...) {
 prophet_xgboost_predict_impl <- function(object, new_data, ...) {
 
     # PREPARE INPUTS
-    prophet_model <- object$models$model_1
-    xgboost_model <- object$models$model_2
-    idx_future    <- new_data %>% timetk::tk_index()
-    xreg_recipe   <- object$extras$xreg_recipe
+    prophet_model   <- object$models$model_1
+    xgboost_model   <- object$models$model_2
+    idx_future      <- new_data %>% timetk::tk_index()
+    xreg_recipe     <- object$extras$xreg_recipe
+    logistic_params <- object$extras$logistic_params
 
     # Construct Future Frame
     df <- tibble::tibble(
         ds = idx_future
     )
 
-    # XREG
-    xreg_tbl <- bake_xreg_recipe(xreg_recipe, new_data, format = "tbl")
+    # Logistic Growth
+    if (logistic_params$growth == "logistic") {
+        df$cap   <- logistic_params$logistic_cap
+        df$floor <- logistic_params$logistic_floor
+    }
 
     # PREDICTIONS
     preds_prophet_df <- stats::predict(prophet_model, df)
@@ -574,6 +644,9 @@ prophet_xgboost_predict_impl <- function(object, new_data, ...) {
     # Return predictions as numeric vector
     preds_prophet <- preds_prophet_df %>% dplyr::pull(yhat)
 
+    # XREG
+    xreg_tbl <- bake_xreg_recipe(xreg_recipe, new_data, format = "tbl")
+
     # xgboost
     if (!is.null(xreg_tbl)) {
         preds_xgboost <- xgboost_predict(xgboost_model, newdata = xreg_tbl, ...)
